# Copyright (c) Microsoft. All rights reserved.
# Licensed under the MIT license. See LICENSE file in the project root for full license information.

from builtins import input

# Using the Python Device SDK for IoT Hub:
#   https://github.com/Azure/azure-iot-sdk-python
# The sample connects to a device-specific MQTT endpoint on your IoT Hub.
# pylint: disable=E0611
from iothub_service_client import IoTHubDeviceMethod, IoTHubError

# The service connection string to authenticate with your IoT hub.
# Using the Azure CLI:
# az iot hub show-connection-string --hub-name {your iot hub name}
CONNECTION_STRING = "HostName=Homevee-Cloud.azure-devices.net;SharedAccessKeyName=iothubowner;SharedAccessKey=8k64yd8mLPj4rCwQP6XnNPiKhbr4tStwSImct2hDXaM="
DEVICE_ID = "9QYWLEVU"

# Details of the direct method to call.
METHOD_NAME = "SetTelemetryInterval"
METHOD_PAYLOAD = "10"
TIMEOUT = 60

def iothub_devicemethod_sample_run():
    try:
        # Connect to your hub.
        iothub_device_method = IoTHubDeviceMethod(CONNECTION_STRING)

        # Call the direct method.
        response = iothub_device_method.invoke(DEVICE_ID, METHOD_NAME, METHOD_PAYLOAD, TIMEOUT)

        print ( "" )
        print ( "Device Method called" )
        print ( "Device Method name       : {0}".format(METHOD_NAME) )
        print ( "Device Method payload    : {0}".format(METHOD_PAYLOAD) )
        print ( "" )
        print ( "Response status          : {0}".format(response.status) )
        print ( "Response payload         : {0}".format(response.payload) )

        input("Press Enter to continue...\n")

    except IoTHubError as iothub_error:
        print ( "" )
        print ( "Unexpected error {0}".format(iothub_error) )
        return
    except KeyboardInterrupt:
        print ( "" )
        print ( "IoTHubDeviceMethod sample stopped" )

if __name__ == '__main__':
    print ( "IoT Hub Python quickstart #2..." )
    print ( "    Connection string = {0}".format(CONNECTION_STRING) )
    print ( "    Device ID         = {0}".format(DEVICE_ID) )

    iothub_devicemethod_sample_run()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database


def get_all_api_key_data(user, db):
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    services = [
        {'servicename': 'Open Weather Map',
         'description': 'Open Weather Map stellt Wetterdaten und -vorhersagen bereit.', 'registerkeyurl': '',
         'servicelogourl': 'https://openweathermap.org/themes/openweathermap/assets/vendor/owm/img/logo_OpenWeatherMap_orange.svg'},
        {'servicename': 'IMDB', 'description': 'IMDB ist eine internationale Filmdatenbank', 'registerkeyurl': '',
         'servicelogourl': 'http://ia.media-imdb.com/images/G/01/imdb/images/mobile/imdb-logo-responsive@2-868559777._CB514893749_.png'}
    ]

    output = []

    for service in services:
        service['key'] = get_api_key(service['servicename'], db)

        output.append(service)

    return {'apikeys': output}

def get_api_key(servicename, db):
    service_data = Database.select_one("SELECT API_KEY FROM API_KEYS WHERE SERVICE_NAME = :name", {'name': servicename}, db)

    try:
        key = service_data['API_KEY']
    except TypeError as e:
        #key doesn't exist
        return None

    return key

def set_api_key(user, service, api_key, db):
    if not user.has_permission("admin"):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    params = {'key': api_key, 'name': service}

    Database.update("UPDATE OR IGNORE API_KEYS SET API_KEY = :key WHERE SERVICE_NAME = :name;", params, db)
    Database.insert("INSERT OR IGNORE INTO API_KEYS (API_KEY, SERVICE_NAME) VALUES (:key, :name);", params, db)

    return Status(type=STATUS_OK).get_dict()


# Copyright (c) Microsoft. All rights reserved.
# Licensed under the MIT license. See LICENSE file in the project root for full license information.

import time

# Using the Python Device SDK for IoT Hub:
#   https://github.com/Azure/azure-iot-sdk-python
# The sample connects to a device-specific MQTT endpoint on your IoT Hub.
# pylint: disable=E0611
from iothub_client import IoTHubClient, IoTHubTransportProvider
from iothub_client import IoTHubError, DeviceMethodReturnValue

# The device connection string to authenticate the device with your IoT hub.
# Using the Azure CLI:
# az iot hub device-identity show-connection-string --hub-name {YourIoTHubName} --device-id MyNodeDevice --output table
HOST_NAME = "Homevee-Cloud.azure-devices.net"
DEVICE_ID = "9QYWLEVU"
SHARED_ACCESS_KEY = "VqpnmyRheRdYhEzMogoWHwX3k4VdXEjL/d4dXJJ9KZM="
CONNECTION_STRING = "HostName="+HOST_NAME+";DeviceId="+DEVICE_ID+";SharedAccessKey="+SHARED_ACCESS_KEY

# Using the MQTT protocol.
PROTOCOL = IoTHubTransportProvider.MQTT
MESSAGE_TIMEOUT = 10000

# Define the JSON message to send to IoT Hub.
TEMPERATURE = 20.0
HUMIDITY = 60
MSG_TXT = "{\"temperature\": %.2f,\"humidity\": %.2f}"

INTERVAL = 1

def send_confirmation_callback(message, result, user_context):
    print ( "IoT Hub responded to message with status: %s" % (result) )

def iothub_client_init():
    # Create an IoT Hub client
    print(CONNECTION_STRING)
    client = IoTHubClient(CONNECTION_STRING, PROTOCOL)
    return client

# Handle direct method calls from IoT Hub
def device_method_callback(method_name, payload, user_context):
    global INTERVAL
    print ( "\nMethod callback called with:\nmethodName = %s\npayload = %s" % (method_name, payload) )
    device_method_return_value = DeviceMethodReturnValue()
    if method_name == "SetTelemetryInterval":
        try:
            INTERVAL = int(payload)
            # Build and send the acknowledgment.
            device_method_return_value.response = "{ \"Response\": \"Executed direct method %s\" }" % method_name
            device_method_return_value.status = 200
        except ValueError:
            # Build and send an error response.
            device_method_return_value.response = "{ \"Response\": \"Invalid parameter\" }"
            device_method_return_value.status = 400
    else:
        # Build and send an error response.
        device_method_return_value.response = "{ \"Response\": \"Direct method not defined: %s\" }" % method_name
        device_method_return_value.status = 404
    return device_method_return_value

def iothub_client_telemetry_sample_run():

    try:
        client = iothub_client_init()
        print ( "IoT Hub device sending periodic messages, press Ctrl-C to exit" )

        # Set up the callback method for direct method calls from the hub.
        client.set_device_method_callback(
            device_method_callback, None)

        while True:
            time.sleep(INTERVAL)

    except IoTHubError as iothub_error:
        print ( "Unexpected error %s from IoTHub" % iothub_error )
        return
    except KeyboardInterrupt:
        print ( "IoTHubClient sample stopped" )

if __name__ == '__main__':
    print ( "IoT Hub Quickstart #2 - Simulated device" )
    print ( "Press Ctrl-C to exit" )
    iothub_client_telemetry_sample_run()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Functions import sensor_data
from Homevee.Functions.sensor_data import SENSOR_TYPE_MAP
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *
from Homevee.VoiceAssistant import Helper
from Homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule


class VoiceDeviceGetSensorDataModule(VoiceDeviceControlModule):
    def get_pattern(self, db):
        SENSOR_ADJECTIVES = ['warm', 'kalt', 'dunkel', 'hell', 'feucht', 'laut']
        SENSOR_SUBJECTIVES = ['temperatur', 'helligkeit', 'luftfeuchtigkeit', 'lautstärke', 'verbrauch',
                              'stromverbrauch']
        return [
            ['wie', SENSOR_ADJECTIVES, 'ist'],
            ['wie', 'ist', ['der', 'die', 'das'], SENSOR_SUBJECTIVES]
        ]

    def get_label(self):
        return "sensor"

    def run_command(self, username, text, context, db):
        return self.get_sensor_data(username, text, context, db)

    def get_context_key(self):
        return "CONTEXT_GET_SENSOR_DATA"

    def get_sensor_data(self, username, text, context, db):
        room = self.find_room(text, db)

        sensor_type = self.find_sensor_type(text)

        if sensor_type is None:
            if context is not None and context:
                sensor_type = context['sensor_type']

        if room is None:
            if context is None:
                answer_data = [
                    ['Welchen Raum ', ['meinst du', 'möchtest du abfragen'], '?']
                ]
                output = Helper.generate_string(answer_data)

                Helper.set_context(username, self.get_context_key(), {'sensor_type': sensor_type}, db)

                return {'msg_speech': output, 'msg_text': output}
            else:
                if 'room' in context:
                    room = context['room']
                    #print(room)

        if sensor_type in SENSOR_TYPE_MAP:
            einheit = SENSOR_TYPE_MAP[sensor_type]['einheit_word']
        else:
            answer_data = [
                ['Diesen Wert kann ich noch nicht abfragen.'],
                ['Dieser Wert kann noch nicht abgefragt werden.']
            ]
            output = Helper.generate_string(answer_data)
            return {'msg_speech': output, 'msg_text': output}


        values = []



        data = [
            {'type': ZWAVE_SENSOR, 'table': 'ZWAVE_SENSOREN', 'id_col': 'ID', 'room_col': 'RAUM', 'type_col': 'SENSOR_TYPE'},
            {'type': MQTT_SENSOR, 'table': 'MQTT_SENSORS', 'id_col': 'ID', 'room_col': 'ROOM', 'type_col': 'TYPE'}
        ]

        for item in data:
            results = Database.select_all("SELECT * FROM "+item['table']+" WHERE "+item['type_col']+" = :type AND "+item['room_col']+" = :room",
                        {'type': sensor_type, 'room': room['LOCATION']}, db)

            for sensor in results:
                values.append(sensor_data.get_sensor_data(username, room['LOCATION'], item['type'], sensor[item['id_col']], False, db))

        if len(values) > 0:
            #compute average value of all sensors in this room
            sum = 0

            try:
                for value in values:
                    sum += float(value)
            except Exception:
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' gibt es noch keine ', ['Messungen', 'Werte', 'Sensorwerte'],'.']
                ]

                output = Helper.generate_string(answer_data)

                return {'msg_speech': output, 'msg_text': output}

            if sensor_type not in ['powermeter']:
                avg_value = sum/len(values)
                value_word = str(avg_value) + ' ' + einheit
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' hat es ' + value_word, '.']
                ]
            else:
                value_word = str(int(sum)) + ' ' + einheit
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' liegt der  ', ['Verbrauch', 'Stromverbrauch'],' bei ', value_word, '.'],
                    ['Der Raum ' + room['NAME'], ' verbraucht aktuell  ', value_word, '.']
                ]

            output = Helper.generate_string(answer_data)

            # helper.set_context(username, GET_SENSOR_DATA, {'room': room}, db)
        else:
            answer_data = [
                ['Ich konnte ', ['hier', 'in diesem Raum'] ,' keine ', ['Daten', 'Sensoren', 'Sensorwerte'], ' finden.']
            ]
            output = Helper.generate_string(answer_data)

        return {'msg_speech': output, 'msg_text': output}

    def get_reed_data(self, username, text, context, db):
        room = self.find_room(text, db)

        output = 'Reed'
        return {'msg_speech': output, 'msg_text': output}

    def get_presence_data(self, username, text, context, db):
        room = self.find_room(text, db)

        output = 'Anwesenheit'
        return {'msg_speech': output, 'msg_text': output}

    def find_sensor_type(self, text):
        words = text.split()

        type_keywords = [
            {'type': 'temp', 'Item': ['warm', 'kalt', 'temperatur']},
            {'type': 'hygro', 'Item': ['feucht', 'luftfeuchtigkeit', 'luftfeuchte']},
            {'type': 'helligkeit', 'Item': ['hell', 'helligkeit']},
            {'type': 'uv', 'Item': ['uv-licht']},
            {'type': 'powermeter', 'Item': ['verbrauch', 'stromverbrauch']}
        ]

        for type_keyword in type_keywords:
            for keyword in type_keyword['Item']:
                if keyword in words:
                    return type_keyword['type']

        return None


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sqlite3
import traceback

from Homevee.DBMigration import DBMigration
from Homevee.Helper import Logger
from Homevee.Utils import Constants


# use encrypted databases
# http://charlesleifer.com/blog/encrypted-sqlite-databases-with-python-and-sqlcipher/

class Database():
    @staticmethod
    def dict_factory(cursor, row):
        d = {}
        for idx, col in enumerate(cursor.description):
            d[col[0]] = row[idx]
        return d

    @staticmethod
    def get_database_con():
        db_path = os.path.join(Constants.DATA_DIR, "data.db")

        # init db if not yet existing
        if not os.path.isfile(db_path):
            open(db_path, "w+")
            Database.upgrade()
        con = sqlite3.connect(db_path)
        con.text_factory = str
        con.row_factory = Database.dict_factory

        return con

    @staticmethod
    def get_server_data(key, db=None):
        item = Database.select_one("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                    {'key': key}, db)

        try:
            return item['VALUE']
        except Exception as e:
            return None

    @staticmethod
    def set_server_data(key, value, db=None):
        try:
            Database.insert("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES(:value, :key)",
                    {'value': value, 'key': key}, db)
        except:
            Database.update("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key",
                    {'value': value, 'key': key}, db)
            
    @staticmethod
    def do_query(query, params, db_con=None):
        #Logger.log(("DATABASE_QUERY", query, params))

        if db_con is None:
            db_con = Database.get_database_con()
        with db_con:
            cur = db_con.cursor()
            cur.execute(query, params)
            return cur

    @staticmethod
    def execute_script(script, db_con=None):
        #Logger.log(("DATABASE_QUERY", query, params))

        if db_con is None:
            db_con = Database.get_database_con()
        with db_con:
            cur = db_con.cursor()
            cur.executescript(script)
        
    @staticmethod
    def select_one(query, params, db_con=None):
        try:
            cur = Database.do_query(query, params, db_con)
            output = cur.fetchone()
            cur.close()
            return output
        except:
            return None
        
    @staticmethod
    def select_all(query, params, db_con=None):
        try:
            cur = Database.do_query(query, params, db_con)
            output = cur.fetchall()
            cur.close()
            return output
        except:
            return []
        
    @staticmethod
    def insert(query, params, db_con=None):
        try:
            cur = Database.do_query(query, params, db_con)
            last_id = cur.lastrowid
            cur.close()
            return last_id
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False
        
    @staticmethod
    def update(query, params, db_con=None):
        try:
            cur = Database.do_query(query, params, db_con)
            cur.close()
            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False
        
    @staticmethod
    def delete(query, params, db_con=None):
        try:
            cur = Database.do_query(query, params, db_con)
            cur.close()
            return True
        except:
            if Logger.IS_DEBUG:
                traceback.print_exc()
        return False
        
    @staticmethod
    def upgrade():
        Logger.log("Upgrading Database...")

        db = Database.get_database_con()

        with db:
            cur = db.cursor()

            migration = DBMigration()

            try:
                db_version = int(Database.get_server_data("DB_VERSION", db))
            except:
                db_version = 0

            db_script_map = migration.get_filecontent_version_map()

            db_script_versions = db_script_map.keys()

            last_version = 0

            for version in db_script_versions:
                if (version <= db_version):
                    continue

                try:
                    Logger.log("Executing DB-Upgrade-Script V"+str(version)+"...")
                    cur.executescript(db_script_map[version])
                except:
                    Database.set_server_data("DB_VERSION", version, db)

            cur.close()


# https://github.com/olucurious/PyFCM
from Homevee.Helper.HomeveeCloud import HomeveeCloudWrapper
from Homevee.Item.User import User
from Homevee.Utils.Database import Database


class NotificationManager():
    def __init__(self):
        return

    def send_notification(self, registration_ids, message_title, message_body, click_action=None):
        cloud_wrapper = HomeveeCloudWrapper()
        message_data = {'title': message_title, 'msg': message_body, 'clickaction': click_action}
        cloud_wrapper.send_push_notification(registration_ids, message_data)

    def send_notification_to_users(self, users, message_title, message_body, db, click_action=None):
        registration_ids = self.get_user_tokens(users, db)
        self.send_notification(registration_ids, message_title, message_body, click_action)

    def send_notification_to_admin(self, message_title, message_body, db, click_action=None):
        admins = User.load_by_permission("admin", db)
        self.send_notification_to_users(admins, message_title, message_body, db, click_action)

    def get_user_tokens(self, users=None, db=None):
            tokens = []

            if users is None or (len(users) is 0):
                cur = db.cursor()

                Database.select_all("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS", db)

                for item in cur.fetchall():
                    tokens.append(item['TOKEN'])
            else:
                cur = db.cursor()

                for user in users:
                    Database.select_all("SELECT TOKEN FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user",
                                {'user': user}, db)

                    for item in cur.fetchall():
                        if (item['TOKEN'] is not None):
                            tokens.append(item['TOKEN'])

            #print users
            #print tokens

            return tokens

    def delete_user_tokens(self, username, db):
            Database.delete("DELETE FROM PUSH_NOTIFICATION_TOKENS WHERE USERNAME = :user", {'user': username}, db)


from Homevee.Exception import AbstractFunctionCallException


class Condition():
    def __init__(self):
        return

    def is_true(self):
        raise AbstractFunctionCallException("Condition.is_true() is abstract")


import json

from Homevee.Helper.HomeveeCloud.CloudAPI import CloudAPI
from Homevee.Helper.HomeveeCloud.Exception import APINotAuthenticatedException, APIErrorException


class HomeveeCloudWrapper():
    def __init__(self, remote_id=None, access_token=None):
        self.cloud_api = CloudAPI(remote_id, access_token)

        self.remote_id = self.cloud_api.remote_id
        self.access_token = self.cloud_api.access_token

    def is_premium(self):
        response = self.cloud_api.do_get("/ispremium/"+self.remote_id, {})
        data = self.check_response(response)
        return data['is_premium']

    def send_push_notification(self, registration_ids, message_data):
        response = self.cloud_api.do_put("/sendnotification",
                                         {'registration_ids': registration_ids,
                                          'message_data': message_data})
        data = self.check_response(response)
        return data['status']

    def set_ip(self, ip):
        response = self.cloud_api.do_put("/setlocalip/"+self.remote_id, {'ip': ip})
        data = self.check_response(response)
        return data['status']

    def set_cert(self, cert):
        response = self.cloud_api.do_put("/setlocalcert/"+self.remote_id, {'cert': cert})
        data = self.check_response(response)
        return data['status']

    def get_ip(self):
        response = self.cloud_api.do_get("/getlocalip/"+self.remote_id, {})
        data = self.check_response(response)
        return data['local_ip']

    def get_cert(self):
        response = self.cloud_api.do_get("/getlocalcert/"+self.remote_id, {})
        data = self.check_response(response)
        return data['local_cert']

    def get_cloud_cert(self, cloud_ip):
        response = self.cloud_api.do_get("/getcloudcert/"+cloud_ip, {})
        data = self.check_response(response)
        return data['cloud_cert']

    def get_used_cloud(self):
        response = self.cloud_api.do_get("/getusedcloud/"+self.remote_id, {})
        data = self.check_response(response)
        return data['cloud']

    def get_cloud_to_use(self):
        response = self.cloud_api.do_get("/getcloudtouse/"+self.remote_id, {})
        data = self.check_response(response)
        return data['cloud']

    def check_response(self, response):
        if response is None:
            raise APIErrorException("API-Call not successful")

        if response.status_code == 401:
            raise APINotAuthenticatedException("Invalid credentials given")

        if response.status_code != 200:
            raise APIErrorException("API-Call not successful")

        return json.loads(response.response)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import ssl
import time
import traceback
from _thread import start_new_thread
from urllib.parse import urlencode
from urllib.request import Request, urlopen

from Homevee.API import API
from Homevee.Helper import Logger, translations
from Homevee.Helper.HomeveeCloud import HomeveeCloudWrapper
from Homevee.Utils import Constants
from Homevee.Utils.Constants import END_OF_MESSAGE
from Homevee.Utils.Database import Database
from .Helper.helper_functions import remote_control_enabled

CLOUD_URL = "premium.cloud.homevee.de"
CLOUD_PORT = 7778

#Verbindung zu Cloud herstellen

#Auf Befehl warten, ausführen und Ergebnis zurück an Server senden

class CloudConnection():
    def __init__(self):
        self.cloud_url = CLOUD_URL
        self.cloud_port = CLOUD_PORT

    def start_connection_loop(self):
        start_new_thread(self.connection_loop, ())

    def connection_loop(self):
        db = Database.get_database_con()
        REMOTE_ID, ACCESS_TOKEN = self.get_remote_data(db)
        #if REMOTE_ID is None or ACCESS_TOKEN is None:
        #    REMOTE_ID, ACCESS_TOKEN = self.register_to_cloud(db)
            # Logger.log("NO CLOUD CONNECTION CREDENTIALS")

        print(translations.translate('your_remote_id_is') + REMOTE_ID)

        while (True):
            self.do_connection()

    def save_cloud_cert(self, cloud_ip):
        cloud_cert = HomeveeCloudWrapper().get_cloud_cert(cloud_ip)

        open(Constants.LOCAL_SSL_CERT, "wt").write(cloud_cert)

    def do_connection(self):
        db = Database.get_database_con()
        conn = None

        if (not remote_control_enabled(db)):
            # Logger.log("REMOTE CONTROL NOT ENABLED")
            # time.sleep(5 * 60)  # 5 Minuten warten
            time.sleep(10)
            return

        cloud_to_use = HomeveeCloudWrapper().get_cloud_to_use()
        Logger.log("Cloud to use: "+str(cloud_to_use))

        if cloud_to_use is not None:
            #Logger.log("using cloud: " + cloud_to_use)
            pass
        else:
            cloud_to_use = 'free.cloud.homevee.de'
            #Logger.log("using default cloud: " + cloud_to_use)
            pass

        try:
            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # conn.settimeout(5*60)
            # conn.connect((CLOUD_URL, CLOUD_PORT))
            # Logger.log("Connected")

            #Neues SSL-Verfahren (Instabil)
            #self.save_cloud_cert(cloud_to_use)
            #conn = ssl.wrap_socket(conn, ca_certs=Constants.CLOUD_SSL_CERT)#,cert_reqs=ssl.CERT_REQUIRED, ssl_version=ssl.PROTOCOL_SSLv23)


            #Altes SSL-Verfahren (Stabil)
            conn = ssl.wrap_socket(conn)

            # Logger.log("Wrapping SSL")
            # Logger.log(("Connecting to cloud ("+CLOUD_URL+")..."))
            conn.connect((cloud_to_use, CLOUD_PORT))
            start_new_thread(self.keep_connection_alive, (conn, 60))
            # print ("SSL connected")
            # Logger.log("Connection successful!")

            REMOTE_ID, ACCESS_TOKEN = self.get_remote_data(db)
            data_dict = {'access_token': ACCESS_TOKEN, 'remote_id': REMOTE_ID}
            data = json.dumps(data_dict) + END_OF_MESSAGE

            len_send = conn.send(bytes(data, 'utf-8'))

            while 1:
                try:
                    data = conn.recv(8192).decode('utf-8')
                except:
                    data = None

                if data is None or data == "":
                    db.close()
                    conn.close()
                    return

                Logger.log("")
                Logger.log(("Received from Cloud: " + data))

                data_parts = data.split(END_OF_MESSAGE)

                for data_part in data_parts:
                    if data_part is None or data_part == "":
                        continue

                    Logger.log("")
                    Logger.log(data_part)

                    data_dict = json.loads(data_part)

                    if 'result' in data_dict:
                        Logger.log(data_dict)

                        if (data_dict['result'] == "error"):
                            Logger.log("couldn't connect to cloud")
                            time.sleep(60)  # 1 Minute warten
                    else:
                        #Logger.log("Processing data:")
                        #Logger.log(data_dict)
                        try:
                            client_id = data_dict['client_id']

                            data_dict = json.loads(data_dict['msg'])

                            msg = API().process_data(data_dict, db)

                            # start_time = time.time()
                            # compressed_message = compression.compress_string(msg)
                            # end_time = time.time()

                            # print "uncompressed: " + str(len(msg))
                            # print "compressed: " + str(len(compressed_message)) + ", time: " + str(end_time - start_time)
                            # compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                            data = {}
                            data['msg'] = msg
                            data['client_id'] = client_id
                            data = json.dumps(data)

                            if data is None:
                                data = {'status': 'error'}
                                data['client_id'] = client_id
                                data = json.dumps(data)
                            # else:
                            #    data['client_id'] = client_id
                            #    data['msg'] = json.dumps(msg)
                            #    data = json.dumps(data)

                            data_to_send = data + END_OF_MESSAGE

                            len_send = conn.send(data_to_send.encode('utf-8'))

                            Logger.log(("Sent to remote Server (" + data_dict['action'] + ")(client: " + str(
                                client_id) + "): " + data))
                        except Exception as e:
                            #if(Logger.IS_DEBUG):
                            #    traceback.print_exc()
                            if 'status' in data_dict:
                                if data_dict['status'] == 'connectionok':
                                    #Logger.log('connectionok')
                                    pass
                                if data_dict['status'] == 'nocredentials':
                                    # Credentials wrong
                                    # deleting them from db
                                    with db:
                                        # Logger.log("wrongcredentials")
                                        time.sleep(60)  # 1 Minute warten
                                        # cur = db.cursor()
                                        # Database.delete("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ID', 'REMOTE_ACCESS_TOKEN')")

        except:
            # do not print errors
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            time.sleep(5)

            if conn is not None:
                conn.close()

        db.close()

    def keep_connection_alive(self, conn, time):
        while True:
            try:
                # Send some data every minute
                time.sleep(time)
                Logger.log("sending ping")
                conn.send(1)
            except:
                break

    # Returns a tuple: (REMOTE_ID, REMOTE_ACCESS_TOKEN)
    def get_remote_data(self, db):
        # Database.delete("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ACCESS_TOKEN', 'REMOTE_ID')", {}, db)

        remote_id = Database.get_server_data("REMOTE_ID", db)
        remote_access_token = Database.get_server_data("REMOTE_ACCESS_TOKEN", db)

        return (remote_id, remote_access_token)

    def register_to_cloud(self, db):
        url = 'https://cloud.homevee.de/server-api.php'  # Set destination URL here
        post_fields = {'action': 'generateremoteid'}  # Set POST fields here

        request = Request(url, urlencode(post_fields).encode())
        response = urlopen(request).read().decode()

        data = json.loads(response)

        remote_id = data['remote_id']
        access_token = data['access_token']

        # Logger.log("Deine Remote-ID lautet: "+remote_id)

        # save remote id
        Database.set_server_data("REMOTE_ID", remote_id, db)
        Database.set_server_data("REMOTE_ACCESS_TOKEN", access_token, db)

        return remote_id, access_token


# Copyright (c) Microsoft. All rights reserved.
# Licensed under the MIT license. See LICENSE file in the project root for full license information.

import random
import time

# Using the Python Device SDK for IoT Hub:
#   https://github.com/Azure/azure-iot-sdk-python
# The sample connects to a device-specific MQTT endpoint on your IoT Hub.
# pylint: disable=E0611
from iothub_client import IoTHubClient, IoTHubTransportProvider
from iothub_client import IoTHubMessage, IoTHubError

# The device connection string to authenticate the device with your IoT hub.
# Using the Azure CLI:
# az iot hub device-identity show-connection-string --hub-name {YourIoTHubName} --device-id MyNodeDevice --output table
CONNECTION_STRING = "HostName=Homevee-Cloud.azure-devices.net;DeviceId=9QYWLEVU;SharedAccessKey=VqpnmyRheRdYhEzMogoWHwX3k4VdXEjL/d4dXJJ9KZM="

# Using the MQTT protocol.
PROTOCOL = IoTHubTransportProvider.MQTT
MESSAGE_TIMEOUT = 10000

# Define the JSON message to send to IoT Hub.
TEMPERATURE = 20.0
HUMIDITY = 60
MSG_TXT = "{\"temperature\": %.2f,\"humidity\": %.2f}"

INTERVAL = 1

def send_confirmation_callback(message, result, user_context):
    print ( "IoT Hub responded to message with status: %s" % (result) )

def iothub_client_init():
    # Create an IoT Hub client
    client = IoTHubClient(CONNECTION_STRING, PROTOCOL)
    return client

def iothub_client_telemetry_sample_run():

    try:
        client = iothub_client_init()
        print ( "IoT Hub device sending periodic messages, press Ctrl-C to exit" )

        while True:
            # Build the message with simulated telemetry values.
            temperature = TEMPERATURE + (random.random() * 15)
            humidity = HUMIDITY + (random.random() * 20)
            msg_txt_formatted = MSG_TXT % (temperature, humidity)
            message = IoTHubMessage(msg_txt_formatted)

            # Add a custom application property to the message.
            # An IoT hub can filter on these properties without access to the message body.
            prop_map = message.properties()
            if temperature > 30:
              prop_map.add("temperatureAlert", "true")
            else:
              prop_map.add("temperatureAlert", "false")

            # Send the message.
            print( "Sending message: %s" % message.get_string() )
            client.send_event_async(message, send_confirmation_callback, None)
            time.sleep(INTERVAL)

    except IoTHubError as iothub_error:
        print ( "Unexpected error %s from IoTHub" % iothub_error )
        return
    except KeyboardInterrupt:
        print ( "IoTHubClient sample stopped" )

if __name__ == '__main__':
    print ( "IoT Hub Quickstart #2 - Simulated device" )
    print ( "Press Ctrl-C to exit" )
    iothub_client_telemetry_sample_run()